@@ -38,6 +38,6 @@ module.exports = class DocumentCache {
       delete this.cache[this.order.shift()];
     }
     this.order.push(url);
-    this.cache[url] = {ctx: ctx, expires: (Date.now() + this.expires)};
+    this.cache[url] = {ctx, expires: (Date.now() + this.expires)};
   }
 };
