@@ -8,7 +8,8 @@ const JsonLdError = require('./JsonLdError');
 const {
   isArray: _isArray,
   isObject: _isObject,
-  isString: _isString
+  isString: _isString,
+  isUndefined: _isUndefined
 } = require('./types');
 
 const {
@@ -100,13 +101,20 @@ api.compact = ({
 
   // use any scoped context on activeProperty
   const ctx = _getContextValue(activeCtx, activeProperty, '@context');
-  if(ctx) {
-    activeCtx = _processContext({activeCtx, localCtx: ctx, options});
+  if(!_isUndefined(ctx)) {
+    // Note: spec's `from term` var is named `isPropertyTermScopedContext`
+    activeCtx = _processContext({
+      activeCtx,
+      localCtx: ctx,
+      isPropertyTermScopedContext: true,
+      options
+    });
   }
 
   // recursively compact object
   if(_isObject(element)) {
-    if(options.link && '@id' in element && element['@id'] in options.link) {
+    if(options.link && '@id' in element &&
+      options.link.hasOwnProperty(element['@id'])) {
       // check for a linked element to reuse
       const linked = options.link[element['@id']];
       for(let i = 0; i < linked.length; ++i) {
@@ -119,10 +127,10 @@ api.compact = ({
     // do value compaction on @values and subject references
     if(_isValue(element) || _isSubjectReference(element)) {
       const rval =
-        api.compactValue({activeCtx, activeProperty, value: element});
+        api.compactValue({activeCtx, activeProperty, value: element, options});
       if(options.link && _isSubjectReference(element)) {
         // store linked element
-        if(!(element['@id'] in options.link)) {
+        if(!(options.link.hasOwnProperty(element['@id']))) {
           options.link[element['@id']] = [];
         }
         options.link[element['@id']].push({expanded: element, compacted: rval});
@@ -135,9 +143,12 @@ api.compact = ({
 
     const rval = {};
 
+    // revert type scoped context
+    activeCtx = activeCtx.revertTypeScopedContext();
+
     if(options.link && '@id' in element) {
       // store linked element
-      if(!(element['@id'] in options.link)) {
+      if(!options.link.hasOwnProperty(element['@id'])) {
         options.link[element['@id']] = [];
       }
       options.link[element['@id']].push({expanded: element, compacted: rval});
@@ -150,14 +161,22 @@ api.compact = ({
     if(types.length > 1) {
       types = Array.from(types).sort();
     }
+    // find all type-scoped contexts based on current context, prior to
+    // updating it
+    const typeContext = activeCtx;
     for(const type of types) {
       const compactedType = api.compactIri(
-        {activeCtx, iri: type, relativeTo: {vocab: true}});
+        {activeCtx: typeContext, iri: type, relativeTo: {vocab: true}});
 
-      // Use any scoped context defined on this value
-      const ctx = _getContextValue(activeCtx, compactedType, '@context');
-      if(ctx) {
-        activeCtx = _processContext({activeCtx, localCtx: ctx, options});
+      // Use any type-scoped context defined on this value
+      const ctx = _getContextValue(typeContext, compactedType, '@context');
+      if(!_isUndefined(ctx)) {
+        activeCtx = _processContext({
+          activeCtx,
+          localCtx: ctx,
+          options,
+          isTypeScopedContext: true
+        });
       }
     }
 
@@ -168,13 +187,16 @@ api.compact = ({
 
       // compact @id and @type(s)
       if(expandedProperty === '@id' || expandedProperty === '@type') {
+        // if using a type-scoped context, resolve type values against previous
+        // context
+        const isType = expandedProperty === '@type';
+        const valueContext = isType ?
+          (activeCtx.previousContext || activeCtx) : activeCtx;
         let compactedValue = _asArray(expandedValue).map(
           expandedIri => api.compactIri({
-            activeCtx,
+            activeCtx: valueContext,
             iri: expandedIri,
-            relativeTo: {
-              vocab: expandedProperty === '@type'
-            }
+            relativeTo: {vocab: isType}
           }));
         if(compactedValue.length === 1) {
           compactedValue = compactedValue[0];
@@ -201,8 +223,8 @@ api.compact = ({
 
         // handle double-reversed properties
         for(const compactedProperty in compactedValue) {
-          if(activeCtx.mappings[compactedProperty] &&
-            activeCtx.mappings[compactedProperty].reverse) {
+          if(activeCtx.mappings.has(compactedProperty) &&
+            activeCtx.mappings.get(compactedProperty).reverse) {
             const value = compactedValue[compactedProperty];
             const container = _getContextValue(
               activeCtx, compactedProperty, '@container') || [];
@@ -290,11 +312,11 @@ api.compact = ({
           relativeTo: {vocab: true},
           reverse: insideReverse
         });
-        const nestProperty = (itemActiveProperty in activeCtx.mappings) ?
-          activeCtx.mappings[itemActiveProperty]['@nest'] : null;
+        const nestProperty = activeCtx.mappings.has(itemActiveProperty) ?
+          activeCtx.mappings.get(itemActiveProperty)['@nest'] : null;
         let nestResult = rval;
         if(nestProperty) {
-          _checkNestProperty(activeCtx, nestProperty);
+          _checkNestProperty(activeCtx, nestProperty, options);
           if(!_isObject(rval[nestProperty])) {
             rval[nestProperty] = {};
           }
@@ -319,11 +341,11 @@ api.compact = ({
 
         // if itemActiveProperty is a @nest property, add values to nestResult,
         // otherwise rval
-        const nestProperty = (itemActiveProperty in activeCtx.mappings) ?
-          activeCtx.mappings[itemActiveProperty]['@nest'] : null;
+        const nestProperty = activeCtx.mappings.has(itemActiveProperty) ?
+          activeCtx.mappings.get(itemActiveProperty)['@nest'] : null;
         let nestResult = rval;
         if(nestProperty) {
-          _checkNestProperty(activeCtx, nestProperty);
+          _checkNestProperty(activeCtx, nestProperty, options);
           if(!_isObject(rval[nestProperty])) {
             rval[nestProperty] = {};
           }
@@ -377,7 +399,7 @@ api.compact = ({
                 relativeTo: {vocab: true}
               })] = expandedItem['@index'];
             }
-          } else if(itemActiveProperty in nestResult) {
+          } else if(nestResult.hasOwnProperty(itemActiveProperty)) {
             // can't use @list container for more than 1 list
             throw new JsonLdError(
               'JSON-LD compact error; property has a "@list" @container ' +
@@ -394,7 +416,7 @@ api.compact = ({
             container.includes('@index') && _isSimpleGraph(expandedItem))) {
             // get or create the map object
             let mapObject;
-            if(itemActiveProperty in nestResult) {
+            if(nestResult.hasOwnProperty(itemActiveProperty)) {
               mapObject = nestResult[itemActiveProperty];
             } else {
               nestResult[itemActiveProperty] = mapObject = {};
@@ -465,7 +487,7 @@ api.compact = ({
           // handle language and index maps
           // get or create the map object
           let mapObject;
-          if(itemActiveProperty in nestResult) {
+          if(nestResult.hasOwnProperty(itemActiveProperty)) {
             mapObject = nestResult[itemActiveProperty];
           } else {
             nestResult[itemActiveProperty] = mapObject = {};
@@ -565,6 +587,12 @@ api.compactIri = ({
     return iri;
   }
 
+  // if context is from a property term scoped context composed with a
+  // type-scoped context, then use the previous context instead
+  if(activeCtx.isPropertyTermScoped && activeCtx.previousContext) {
+    activeCtx = activeCtx.previousContext;
+  }
+
   const inverseCtx = activeCtx.getInverse();
 
   // if term is a keyword, it may be compacted to a simple alias
@@ -729,7 +757,7 @@ api.compactIri = ({
       if(iri.indexOf(vocab) === 0 && iri !== vocab) {
         // use suffix as relative iri if it is not a term in the active context
         const suffix = iri.substr(vocab.length);
-        if(!(suffix in activeCtx.mappings)) {
+        if(!activeCtx.mappings.has(suffix)) {
           return suffix;
         }
       }
@@ -760,9 +788,9 @@ api.compactIri = ({
       // 2. value is null, which means we're not compacting an @value, AND
       //   the mapping matches the IRI
       const curie = term + ':' + iri.substr(entry.iri.length);
-      const isUsableCurie = (activeCtx.mappings[term]._prefix &&
-        (!(curie in activeCtx.mappings) ||
-        (value === null && activeCtx.mappings[curie]['@id'] === iri)));
+      const isUsableCurie = (activeCtx.mappings.get(term)._prefix &&
+        (!activeCtx.mappings.has(curie) ||
+        (value === null && activeCtx.mappings.get(curie)['@id'] === iri)));
 
       // select curie if it is shorter or the same length but lexicographically
       // less than the current choice
@@ -794,10 +822,11 @@ api.compactIri = ({
  * @param activeCtx the active context.
  * @param activeProperty the active property that points to the value.
  * @param value the value to compact.
+ * @param {Object} [options] - processing options.
  *
  * @return the compaction result.
  */
-api.compactValue = ({activeCtx, activeProperty, value}) => {
+api.compactValue = ({activeCtx, activeProperty, value, options}) => {
   // value is a @value
   if(_isValue(value)) {
     // get context rules
@@ -826,8 +855,8 @@ api.compactValue = ({activeCtx, activeProperty, value}) => {
       (keyCount === 2 && '@index' in value && !preserveIndex));
     const hasDefaultLanguage = ('@language' in activeCtx);
     const isValueString = _isString(value['@value']);
-    const hasNullMapping = (activeCtx.mappings[activeProperty] &&
-      activeCtx.mappings[activeProperty]['@language'] === null);
+    const hasNullMapping = (activeCtx.mappings.has(activeProperty) &&
+      activeCtx.mappings.get(activeProperty)['@language'] === null);
     if(isValueOnlyKey &&
       (!hasDefaultLanguage || !isValueString || hasNullMapping)) {
       return value['@value'];
@@ -872,7 +901,8 @@ api.compactValue = ({activeCtx, activeProperty, value}) => {
   }
 
   // value is a subject reference
-  const expandedProperty = _expandIri(activeCtx, activeProperty, {vocab: true});
+  const expandedProperty = _expandIri(activeCtx, activeProperty, {vocab: true},
+    options);
   const type = _getContextValue(activeCtx, activeProperty, '@type');
   const compacted = api.compactIri(
     {activeCtx, iri: value['@id'], relativeTo: {vocab: type === '@vocab'}});
@@ -939,9 +969,9 @@ api.removePreserve = (ctx, input, options) => {
       iri: '@id',
       relativeTo: {vocab: true}
     });
-    if(idAlias in input) {
+    if(input.hasOwnProperty(idAlias)) {
       const id = input[idAlias];
-      if(id in options.link) {
+      if(options.link.hasOwnProperty(id)) {
         const idx = options.link[id].indexOf(input);
         if(idx !== -1) {
           // already visited
@@ -1011,9 +1041,9 @@ function _selectTerm(
     // try to compact value to a term
     const term = api.compactIri(
       {activeCtx, iri: value['@id'], relativeTo: {vocab: true}});
-    if(term in activeCtx.mappings &&
-      activeCtx.mappings[term] &&
-      activeCtx.mappings[term]['@id'] === value['@id']) {
+    if(activeCtx.mappings.has(term) &&
+      activeCtx.mappings.get(term) &&
+      activeCtx.mappings.get(term)['@id'] === value['@id']) {
       // prefer @vocab
       prefs.push.apply(prefs, ['@vocab', '@id']);
     } else {
@@ -1055,9 +1085,10 @@ function _selectTerm(
  *
  * @param activeCtx the active context.
  * @param nestProperty a term in the active context or `@nest`.
+ * @param {Object} [options] - processing options.
  */
-function _checkNestProperty(activeCtx, nestProperty) {
-  if(_expandIri(activeCtx, nestProperty, {vocab: true}) !== '@nest') {
+function _checkNestProperty(activeCtx, nestProperty, options) {
+  if(_expandIri(activeCtx, nestProperty, {vocab: true}, options) !== '@nest') {
     throw new JsonLdError(
       'JSON-LD compact error; nested property must have an @nest value ' +
       'resolving to @nest.',
