@@ -42,34 +42,114 @@ api.cache = new ActiveContextCache();
  * @param activeCtx the current active context.
  * @param localCtx the local context to process.
  * @param options the context processing options.
+ * @param isPropertyTermScopedContext `true` if `localCtx` is a scoped context
+ *   from a property term.
+ * @param isTypeScopedContext `true` if `localCtx` is a scoped context
+ *   from a type.
  *
  * @return the new active context.
  */
-api.process = ({activeCtx, localCtx, options}) => {
+api.process = ({
+  activeCtx, localCtx, options,
+  isPropertyTermScopedContext = false,
+  isTypeScopedContext = false
+}) => {
   // normalize local context to an array of @context objects
   if(_isObject(localCtx) && '@context' in localCtx &&
     _isArray(localCtx['@context'])) {
     localCtx = localCtx['@context'];
   }
-  const ctxs = _isArray(localCtx) ? localCtx : [localCtx];
+  const ctxs = _asArray(localCtx);
 
   // no contexts in array, return current active context w/o changes
   if(ctxs.length === 0) {
     return activeCtx;
   }
 
+  // track the previous context
+  const previousContext = activeCtx.previousContext || activeCtx;
+
+  // if context is property scoped and there's a previous context, amend it,
+  // not the current one
+  if(isPropertyTermScopedContext && activeCtx.previousContext) {
+    // TODO: consider optimizing to a shallow copy
+    activeCtx = activeCtx.clone();
+    activeCtx.isPropertyTermScoped = true;
+    activeCtx.previousContext = api.process({
+      activeCtx: activeCtx.previousContext,
+      localCtx: ctxs,
+      options,
+      isPropertyTermScopedContext
+    });
+    return activeCtx;
+  }
+
   // process each context in order, update active context
   // on each iteration to ensure proper caching
   let rval = activeCtx;
   for(let i = 0; i < ctxs.length; ++i) {
     let ctx = ctxs[i];
 
+    // update active context to one computed from last iteration
+    activeCtx = rval;
+
     // reset to initial context
     if(ctx === null) {
-      rval = activeCtx = api.getInitialContext(options);
+      // We can't nullify if there are protected terms and we're
+      // not processing a property term scoped context
+      if(!isPropertyTermScopedContext &&
+        Object.keys(activeCtx.protected).length !== 0) {
+        const protectedMode = (options && options.protectedMode) || 'error';
+        if(protectedMode === 'error') {
+          throw new JsonLdError(
+            'Tried to nullify a context with protected terms outside of ' +
+            'a term definition.',
+            'jsonld.SyntaxError',
+            {code: 'invalid context nullification'});
+        } else if(protectedMode === 'warn') {
+          // FIXME: remove logging and use a handler
+          console.warn('WARNING: invalid context nullification');
+          const oldActiveCtx = activeCtx;
+          // copy all protected term definitions to fresh initial context
+          rval = activeCtx = api.getInitialContext(options).clone();
+          for(const [term, _protected] of
+            Object.entries(oldActiveCtx.protected)) {
+            if(_protected) {
+              activeCtx.mappings[term] =
+                util.clone(oldActiveCtx.mappings[term]);
+            }
+          }
+          activeCtx.protected = util.clone(oldActiveCtx.protected);
+
+          // cache result
+          if(api.cache) {
+            api.cache.set(oldActiveCtx, ctx, rval);
+          }
+
+          continue;
+        }
+        throw new JsonLdError(
+          'Invalid protectedMode.',
+          'jsonld.SyntaxError',
+          {code: 'invalid protected mode', context: localCtx, protectedMode});
+      }
+      rval = activeCtx = api.getInitialContext(options).clone();
+      // if context is type-scoped, ensure previous context has been set
+      if(isTypeScopedContext) {
+        rval.previousContext = previousContext.clone();
+      }
       continue;
     }
 
+    // get context from cache if available
+    if(api.cache) {
+      const cached = api.cache.get(activeCtx, ctx);
+      if(cached) {
+        rval = activeCtx = cached;
+        continue;
+      }
+    }
+
     // dereference @context key if present
     if(_isObject(ctx) && '@context' in ctx) {
       ctx = ctx['@context'];
@@ -82,21 +162,14 @@ api.process = ({activeCtx, localCtx, options}) => {
         'jsonld.SyntaxError', {code: 'invalid local context', context: ctx});
     }
 
-    // get context from cache if available
-    if(api.cache) {
-      const cached = api.cache.get(activeCtx, ctx);
-      if(cached) {
-        rval = activeCtx = cached;
-        continue;
-      }
-    }
+    // TODO: there is likely a `preivousContext` cloning optimization that
+    // could be applied here (no need to copy it under certain conditions)
 
-    // update active context and clone new one before updating
-    activeCtx = rval;
+    // clone context before updating it
     rval = rval.clone();
 
     // define context mappings for keys in local context
-    const defined = {};
+    const defined = new Map();
 
     // handle @version
     if('@version' in ctx) {
@@ -116,7 +189,7 @@ api.process = ({activeCtx, localCtx, options}) => {
       }
       rval.processingMode = 'json-ld-1.1';
       rval['@version'] = ctx['@version'];
-      defined['@version'] = true;
+      defined.set('@version', true);
     }
 
     // if not set explicitly, set processingMode to "json-ld-1.0"
@@ -141,7 +214,7 @@ api.process = ({activeCtx, localCtx, options}) => {
       }
 
       rval['@base'] = base;
-      defined['@base'] = true;
+      defined.set('@base', true);
     }
 
     // handle @vocab
@@ -162,7 +235,7 @@ api.process = ({activeCtx, localCtx, options}) => {
       } else {
         rval['@vocab'] = value;
       }
-      defined['@vocab'] = true;
+      defined.set('@vocab', true);
     }
 
     // handle @language
@@ -179,12 +252,24 @@ api.process = ({activeCtx, localCtx, options}) => {
       } else {
         rval['@language'] = value.toLowerCase();
       }
-      defined['@language'] = true;
+      defined.set('@language', true);
     }
 
+    // handle @protected; determine whether this sub-context is declaring
+    // all its terms to be "protected" (exceptions can be made on a
+    // per-definition basis)
+    defined.set('@protected', ctx['@protected'] || false);
+
     // process all other keys
     for(const key in ctx) {
-      api.createTermDefinition(rval, ctx, key, defined);
+      api.createTermDefinition(
+        rval, ctx, key, defined, options,
+        isPropertyTermScopedContext);
+    }
+
+    // if context is type-scoped, ensure previous context has been set
+    if(isTypeScopedContext && !rval.previousContext) {
+      rval.previousContext = previousContext.clone();
     }
 
     // cache result
@@ -204,11 +289,19 @@ api.process = ({activeCtx, localCtx, options}) => {
  * @param term the term in the local context to define the mapping for.
  * @param defined a map of defining/defined keys to detect cycles and prevent
  *          double definitions.
+ * @param {Object} [options] - creation options.
+ * @param {string} [options.protectedMode="error"] - "error" to throw error
+ *   on `@protected` constraint violation, "warn" to allow violations and
+ *   signal a warning.
+ * @param isPropertyTermScopedContext `true` if `localCtx` is a scoped context
+ *   from a property term.
  */
-api.createTermDefinition = (activeCtx, localCtx, term, defined) => {
-  if(term in defined) {
+api.createTermDefinition = (
+  activeCtx, localCtx, term, defined, options,
+  isPropertyTermScopedContext = false) => {
+  if(defined.has(term)) {
     // term already defined
-    if(defined[term]) {
+    if(defined.get(term)) {
       return;
     }
     // cycle detected
@@ -219,7 +312,7 @@ api.createTermDefinition = (activeCtx, localCtx, term, defined) => {
   }
 
   // now defining term
-  defined[term] = false;
+  defined.set(term, false);
 
   if(api.isKeyword(term)) {
     throw new JsonLdError(
@@ -235,18 +328,24 @@ api.createTermDefinition = (activeCtx, localCtx, term, defined) => {
       {code: 'invalid term definition', context: localCtx});
   }
 
+  // keep reference to previous mapping for potential `@protected` check
+  const previousMapping = activeCtx.mappings.get(term);
+
   // remove old mapping
-  if(activeCtx.mappings[term]) {
-    delete activeCtx.mappings[term];
+  if(activeCtx.mappings.has(term)) {
+    activeCtx.mappings.delete(term);
   }
 
   // get context term value
-  let value = localCtx[term];
+  let value;
+  if(localCtx.hasOwnProperty(term)) {
+    value = localCtx[term];
+  }
 
   // clear context entry
   if(value === null || (_isObject(value) && value['@id'] === null)) {
-    activeCtx.mappings[term] = null;
-    defined[term] = true;
+    activeCtx.mappings.set(term, null);
+    defined.set(term, true);
     return;
   }
 
@@ -266,7 +365,8 @@ api.createTermDefinition = (activeCtx, localCtx, term, defined) => {
   }
 
   // create new mapping
-  const mapping = activeCtx.mappings[term] = {};
+  const mapping = {};
+  activeCtx.mappings.set(term, mapping);
   mapping.reverse = false;
 
   // make sure term definition only has expected keywords
@@ -274,7 +374,7 @@ api.createTermDefinition = (activeCtx, localCtx, term, defined) => {
 
   // JSON-LD 1.1 support
   if(api.processingMode(activeCtx, 1.1)) {
-    validKeys.push('@context', '@nest', '@prefix');
+    validKeys.push('@context', '@nest', '@prefix', '@protected');
   }
 
   for(const kw in value) {
@@ -312,8 +412,9 @@ api.createTermDefinition = (activeCtx, localCtx, term, defined) => {
     }
 
     // expand and add @id mapping
-    const id = api.expandIri(
-      activeCtx, reverse, {vocab: true, base: false}, localCtx, defined);
+    const id = _expandIri(
+      activeCtx, reverse, {vocab: true, base: false}, localCtx, defined,
+      options);
     if(!_isAbsoluteIri(id)) {
       throw new JsonLdError(
         'Invalid JSON-LD syntax; a @context @reverse value must be an ' +
@@ -332,8 +433,8 @@ api.createTermDefinition = (activeCtx, localCtx, term, defined) => {
     }
     if(id !== term) {
       // expand and add @id mapping
-      id = api.expandIri(
-        activeCtx, id, {vocab: true, base: false}, localCtx, defined);
+      id = _expandIri(
+        activeCtx, id, {vocab: true, base: false}, localCtx, defined, options);
       if(!_isAbsoluteIri(id) && !api.isKeyword(id)) {
         throw new JsonLdError(
           'Invalid JSON-LD syntax; a @context @id value must be an ' +
@@ -353,15 +454,15 @@ api.createTermDefinition = (activeCtx, localCtx, term, defined) => {
     // see if the term has a prefix
     if(mapping._termHasColon) {
       const prefix = term.substr(0, colon);
-      if(prefix in localCtx) {
+      if(localCtx.hasOwnProperty(prefix)) {
         // define parent prefix
-        api.createTermDefinition(activeCtx, localCtx, prefix, defined);
+        api.createTermDefinition(activeCtx, localCtx, prefix, defined, options);
       }
 
-      if(activeCtx.mappings[prefix]) {
+      if(activeCtx.mappings.has(prefix)) {
         // set @id based on prefix parent
         const suffix = term.substr(colon + 1);
-        mapping['@id'] = activeCtx.mappings[prefix]['@id'] + suffix;
+        mapping['@id'] = activeCtx.mappings.get(prefix)['@id'] + suffix;
       } else {
         // term is an absolute IRI
         mapping['@id'] = term;
@@ -379,22 +480,30 @@ api.createTermDefinition = (activeCtx, localCtx, term, defined) => {
     }
   }
 
+  // Handle term protection
+  if(value['@protected'] === true ||
+    (defined.get('@protected') === true && value['@protected'] !== false)) {
+    activeCtx.protected[term] = true;
+    mapping.protected = true;
+  }
+
   // IRI mapping now defined
-  defined[term] = true;
+  defined.set(term, true);
 
   if('@type' in value) {
     let type = value['@type'];
     if(!_isString(type)) {
       throw new JsonLdError(
-        'Invalid JSON-LD syntax; an @context @type values must be a string.',
+        'Invalid JSON-LD syntax; an @context @type value must be a string.',
         'jsonld.SyntaxError',
         {code: 'invalid type mapping', context: localCtx});
     }
 
     if(type !== '@id' && type !== '@vocab') {
       // expand @type to full IRI
-      type = api.expandIri(
-        activeCtx, type, {vocab: true, base: false}, localCtx, defined);
+      type = _expandIri(
+        activeCtx, type, {vocab: true, base: false}, localCtx, defined,
+        options);
       if(!_isAbsoluteIri(type)) {
         throw new JsonLdError(
           'Invalid JSON-LD syntax; an @context @type value must be an ' +
@@ -404,7 +513,7 @@ api.createTermDefinition = (activeCtx, localCtx, term, defined) => {
       }
       if(type.indexOf('_:') === 0) {
         throw new JsonLdError(
-          'Invalid JSON-LD syntax; an @context @type values must be an IRI, ' +
+          'Invalid JSON-LD syntax; an @context @type value must be an IRI, ' +
           'not a blank node identifier.',
           'jsonld.SyntaxError',
           {code: 'invalid type mapping', context: localCtx});
@@ -543,6 +652,52 @@ api.createTermDefinition = (activeCtx, localCtx, term, defined) => {
       'Invalid JSON-LD syntax; @context and @preserve cannot be aliased.',
       'jsonld.SyntaxError', {code: 'invalid keyword alias', context: localCtx});
   }
+
+  // FIXME if(1.1) ... ?
+  if(previousMapping && previousMapping.protected &&
+    !isPropertyTermScopedContext) {
+    // force new term to continue to be protected and see if the mappings would
+    // be equal
+    activeCtx.protected[term] = true;
+    mapping.protected = true;
+    if(!_deepCompare(previousMapping, mapping)) {
+      const protectedMode = (options && options.protectedMode) || 'error';
+      if(protectedMode === 'error') {
+        throw new JsonLdError(
+          'Invalid JSON-LD syntax; tried to redefine a protected term.',
+          'jsonld.SyntaxError',
+          {code: 'protected term redefinition', context: localCtx, term});
+      } else if(protectedMode === 'warn') {
+        // FIXME: remove logging and use a handler
+        console.warn('WARNING: protected term redefinition', {term});
+        return;
+      }
+      throw new JsonLdError(
+        'Invalid protectedMode.',
+        'jsonld.SyntaxError',
+        {code: 'invalid protected mode', context: localCtx, term,
+          protectedMode});
+    }
+  }
+};
+
+/**
+ * Expands a string to a full IRI. The string may be a term, a prefix, a
+ * relative IRI, or an absolute IRI. The associated absolute IRI will be
+ * returned.
+ *
+ * @param activeCtx the current active context.
+ * @param value the string to expand.
+ * @param relativeTo options for how to resolve relative IRIs:
+ *          base: true to resolve against the base IRI, false not to.
+ *          vocab: true to concatenate after @vocab, false not to.
+ * @param {Object} [options] - processing options.
+ *
+ * @return the expanded value.
+ */
+api.expandIri = (activeCtx, value, relativeTo, options) => {
+  return _expandIri(activeCtx, value, relativeTo, undefined, undefined,
+    options);
 };
 
 /**
@@ -559,23 +714,31 @@ api.createTermDefinition = (activeCtx, localCtx, term, defined) => {
  *          during context processing).
  * @param defined a map for tracking cycles in context definitions (only given
  *          if called during context processing).
+ * @param {Object} [options] - processing options.
  *
  * @return the expanded value.
  */
-api.expandIri = (activeCtx, value, relativeTo, localCtx, defined) => {
+function _expandIri(activeCtx, value, relativeTo, localCtx, defined, options) {
   // already expanded
   if(value === null || !_isString(value) || api.isKeyword(value)) {
     return value;
   }
 
   // define term dependency if not defined
-  if(localCtx && value in localCtx && defined[value] !== true) {
-    api.createTermDefinition(activeCtx, localCtx, value, defined);
+  if(localCtx && localCtx.hasOwnProperty(value) &&
+    defined.get(value) !== true) {
+    api.createTermDefinition(activeCtx, localCtx, value, defined, options);
+  }
+
+  // if context is from a property term scoped context composed with a
+  // type-scoped context, then use previous context instead
+  if(activeCtx.isPropertyTermScoped && activeCtx.previousContext) {
+    activeCtx = activeCtx.previousContext;
   }
 
   relativeTo = relativeTo || {};
   if(relativeTo.vocab) {
-    const mapping = activeCtx.mappings[value];
+    const mapping = activeCtx.mappings.get(value);
 
     // value is explicitly ignored with a null mapping
     if(mapping === null) {
@@ -601,13 +764,13 @@ api.expandIri = (activeCtx, value, relativeTo, localCtx, defined) => {
     }
 
     // prefix dependency not defined, define it
-    if(localCtx && prefix in localCtx) {
-      api.createTermDefinition(activeCtx, localCtx, prefix, defined);
+    if(localCtx && localCtx.hasOwnProperty(prefix)) {
+      api.createTermDefinition(activeCtx, localCtx, prefix, defined, options);
     }
 
     // use mapping if prefix is defined
-    const mapping = activeCtx.mappings[prefix];
-    if(mapping) {
+    if(activeCtx.mappings.has(prefix)) {
+      const mapping = activeCtx.mappings.get(prefix);
       return mapping['@id'] + suffix;
     }
 
@@ -626,7 +789,7 @@ api.expandIri = (activeCtx, value, relativeTo, localCtx, defined) => {
   }
 
   return value;
-};
+}
 
 /**
  * Gets the initial context.
@@ -647,10 +810,12 @@ api.getInitialContext = options => {
   const initialContext = {
     '@base': base,
     processingMode: options.processingMode,
-    mappings: {},
+    mappings: new Map(),
     inverse: null,
     getInverse: _createInverseContext,
-    clone: _cloneActiveContext
+    clone: _cloneActiveContext,
+    revertTypeScopedContext: _revertTypeScopedContext,
+    protected: {}
   };
   // TODO: consider using LRU cache instead
   if(INITIAL_CONTEXT_CACHE.size === INITIAL_CONTEXT_CACHE_MAX_SIZE) {
@@ -686,10 +851,9 @@ api.getInitialContext = options => {
     // create term selections for each mapping in the context, ordered by
     // shortest and then lexicographically least
     const mappings = activeCtx.mappings;
-    const terms = Object.keys(mappings).sort(_compareShortestLeast);
-    for(let i = 0; i < terms.length; ++i) {
-      const term = terms[i];
-      const mapping = mappings[term];
+    const terms = [...mappings.keys()].sort(_compareShortestLeast);
+    for(const term of terms) {
+      const mapping = mappings.get(term);
       if(mapping === null) {
         continue;
       }
@@ -699,8 +863,7 @@ api.getInitialContext = options => {
 
       // iterate over every IRI in the mapping
       const ids = _asArray(mapping['@id']);
-      for(let ii = 0; ii < ids.length; ++ii) {
-        const iri = ids[ii];
+      for(const iri of ids) {
         let entry = inverse[iri];
         const isKeyword = api.isKeyword(iri);
 
@@ -779,17 +942,17 @@ api.getInitialContext = options => {
 
     let iri;
     let letter;
-    for(let i = 0; i < entries.length; ++i) {
-      iri = entries[i].iri;
+    for(const entry of entries) {
+      iri = entry.iri;
       if(idx >= iri.length) {
         letter = '';
       } else {
         letter = iri[idx];
       }
       if(letter in next) {
-        next[letter].push(entries[i]);
+        next[letter].push(entry);
       } else {
-        next[letter] = [entries[i]];
+        next[letter] = [entry];
       }
     }
 
@@ -809,7 +972,7 @@ api.getInitialContext = options => {
    * @param typeOrLanguageValue the key in the entry to add to.
    */
   function _addPreferredTerm(term, entry, typeOrLanguageValue) {
-    if(!(typeOrLanguageValue in entry)) {
+    if(!entry.hasOwnProperty(typeOrLanguageValue)) {
       entry[typeOrLanguageValue] = term;
     }
   }
@@ -826,6 +989,12 @@ api.getInitialContext = options => {
     child.clone = this.clone;
     child.inverse = null;
     child.getInverse = this.getInverse;
+    child.protected = util.clone(this.protected);
+    if(this.previousContext) {
+      child.isPropertyTermScoped = this.previousContext.isPropertyTermScoped;
+      child.previousContext = this.previousContext.clone();
+    }
+    child.revertTypeScopedContext = this.revertTypeScopedContext;
     if('@language' in this) {
       child['@language'] = this['@language'];
     }
@@ -834,44 +1003,61 @@ api.getInitialContext = options => {
     }
     return child;
   }
+
+  /**
+   * Reverts any type-scoped context in this active context to the previous
+   * context.
+   */
+  function _revertTypeScopedContext() {
+    if(!this.previousContext) {
+      return this;
+    }
+    return this.previousContext.clone();
+  }
 };
 
 /**
  * Gets the value for the given active context key and type, null if none is
- * set.
+ * set or undefined if none is set and type is '@context'.
  *
  * @param ctx the active context.
  * @param key the context key.
  * @param [type] the type of value to get (eg: '@id', '@type'), if not
  *          specified gets the entire entry for a key, null if not found.
  *
- * @return the value.
+ * @return the value, null, or undefined.
  */
 api.getContextValue = (ctx, key, type) => {
-  // return null for invalid key
+  // invalid key
   if(key === null) {
+    if(type === '@context') {
+      return undefined;
+    }
     return null;
   }
 
   // get specific entry information
-  if(ctx.mappings[key]) {
-    const entry = ctx.mappings[key];
+  if(ctx.mappings.has(key)) {
+    const entry = ctx.mappings.get(key);
 
     if(_isUndefined(type)) {
       // return whole entry
       return entry;
     }
-    if(type in entry) {
+    if(entry.hasOwnProperty(type)) {
       // return entry value for type
       return entry[type];
     }
   }
 
   // get default language
-  if(type === '@language' && (type in ctx)) {
+  if(type === '@language' && ctx.hasOwnProperty(type)) {
     return ctx[type];
   }
 
+  if(type === '@context') {
+    return undefined;
+  }
   return null;
 };
 
@@ -935,6 +1121,7 @@ api.isKeyword = v => {
     case '@omitDefault':
     case '@prefix':
     case '@preserve':
+    case '@protected':
     case '@requireAll':
     case '@reverse':
     case '@set':
@@ -1132,3 +1319,48 @@ function _findContextUrls(input, urls, replace, base) {
     }
   }
 }
+
+function _deepCompare(x1, x2) {
+  // compare `null` or primitive types directly
+  if((!(x1 && typeof x1 === 'object')) ||
+     (!(x2 && typeof x2 === 'object'))) {
+    return x1 === x2;
+  }
+  // x1 and x2 are objects (also potentially arrays)
+  const x1Array = Array.isArray(x1);
+  if(x1Array !== Array.isArray(x2)) {
+    return false;
+  }
+  if(x1Array) {
+    if(x1.length !== x2.length) {
+      return false;
+    }
+    for(let i = 0; i < x1.length; ++i) {
+      if(!_deepCompare(x1[i], x2[i])) {
+        return false;
+      }
+    }
+    return true;
+  }
+  // x1 and x2 are non-array objects
+  const k1s = Object.keys(x1);
+  const k2s = Object.keys(x2);
+  if(k1s.length !== k2s.length) {
+    return false;
+  }
+  for(const k1 in x1) {
+    let v1 = x1[k1];
+    let v2 = x2[k1];
+    // special case: `@container` can be in any order
+    if(k1 === '@container') {
+      if(Array.isArray(v1) && Array.isArray(v2)) {
+        v1 = v1.slice().sort();
+        v2 = v2.slice().sort();
+      }
+    }
+    if(!_deepCompare(v1, v2)) {
+      return false;
+    }
+  }
+  return true;
+}
