@@ -9,7 +9,8 @@ const {
   isArray: _isArray,
   isObject: _isObject,
   isEmptyObject: _isEmptyObject,
-  isString: _isString
+  isString: _isString,
+  isUndefined: _isUndefined
 } = require('./types');
 
 const {
@@ -49,6 +50,11 @@ module.exports = api;
  * @param element the element to expand.
  * @param options the expansion options.
  * @param insideList true if the element is a list, false if not.
+ * @param insideIndex true if the element is inside an index container,
+ *          false if not.
+ * @param typeScopedContext an optional type-scoped active context for
+ *          expanding values of nodes that were expressed according to
+ *          a type-scoped context.
  * @param expansionMap(info) a function that can be used to custom map
  *          unmappable values (or to throw an error when they are detected);
  *          if this function returns `undefined` then the default behavior
@@ -62,6 +68,8 @@ api.expand = ({
   element,
   options = {},
   insideList = false,
+  insideIndex = false,
+  typeScopedContext = null,
   expansionMap = () => undefined
 }) => {
   // nothing to expand
@@ -77,7 +85,8 @@ api.expand = ({
   if(!_isArray(element) && !_isObject(element)) {
     // drop free-floating scalars that are not in lists unless custom mapped
     if(!insideList && (activeProperty === null ||
-      _expandIri(activeCtx, activeProperty, {vocab: true}) === '@graph')) {
+      _expandIri(activeCtx, activeProperty, {vocab: true},
+        options) === '@graph')) {
       // TODO: use `await` to support async
       const mapped = expansionMap({
         unmappedValue: element,
@@ -93,7 +102,7 @@ api.expand = ({
     }
 
     // expand element according to value expansion rules
-    return _expandValue({activeCtx, activeProperty, value: element});
+    return _expandValue({activeCtx, activeProperty, value: element, options});
   }
 
   // recursively expand array
@@ -109,7 +118,9 @@ api.expand = ({
         activeProperty,
         element: element[i],
         options,
-        expansionMap
+        expansionMap,
+        insideIndex,
+        typeScopedContext
       });
       if(insideList && (_isArray(e) || _isList(e))) {
         // lists of lists are illegal
@@ -146,36 +157,73 @@ api.expand = ({
 
   // recursively expand object:
 
+  // first, expand the active property
+  const expandedActiveProperty = _expandIri(
+    activeCtx, activeProperty, {vocab: true}, options);
+
+  // second, determine if any type-scoped context should be reverted; it
+  // should only be reverted when the following are all true:
+  // 1. `element` is not a value or subject reference
+  // 2. `insideIndex` is false
+  typeScopedContext = typeScopedContext ||
+    (activeCtx.previousContext ? activeCtx : null);
+  let keys = Object.keys(element).sort();
+  let mustRevert = !insideIndex;
+  if(mustRevert && typeScopedContext && keys.length <= 2 &&
+    !keys.includes('@context')) {
+    for(const key of keys) {
+      const expandedProperty = _expandIri(
+        typeScopedContext, key, {vocab: true}, options);
+      if(expandedProperty === '@value') {
+        // value found, ensure type-scoped context is used to expand it
+        mustRevert = false;
+        activeCtx = typeScopedContext;
+        break;
+      }
+      if(expandedProperty === '@id' && keys.length === 1) {
+        // subject reference found, do not revert
+        mustRevert = false;
+        break;
+      }
+    }
+  }
+
+  if(mustRevert) {
+    // revert type scoped context
+    activeCtx = activeCtx.revertTypeScopedContext();
+  }
+
   // if element has a context, process it
   if('@context' in element) {
     activeCtx = _processContext(
       {activeCtx, localCtx: element['@context'], options});
   }
 
-  // look for scoped context on @type
-  let keys = Object.keys(element).sort();
+  // look for scoped contexts on `@type`
   for(const key of keys) {
-    const expandedProperty = _expandIri(activeCtx, key, {vocab: true});
+    const expandedProperty = _expandIri(activeCtx, key, {vocab: true}, options);
     if(expandedProperty === '@type') {
-      // set scopped contexts from @type
+      // set scoped contexts from @type
       // avoid sorting if possible
       const value = element[key];
       const types =
         Array.isArray(value) ?
           (value.length > 1 ? value.slice().sort() : value) : [value];
       for(const type of types) {
-        const ctx = _getContextValue(activeCtx, type, '@context');
-        if(ctx) {
-          activeCtx = _processContext({activeCtx, localCtx: ctx, options});
+        const ctx = _getContextValue(
+          activeCtx.previousContext || activeCtx, type, '@context');
+        if(!_isUndefined(ctx)) {
+          activeCtx = _processContext({
+            activeCtx,
+            localCtx: ctx,
+            options,
+            isTypeScopedContext: true
+          });
         }
       }
     }
   }
 
-  // expand the active property
-  const expandedActiveProperty = _expandIri(
-    activeCtx, activeProperty, {vocab: true});
-
   // process each key and value in element, ignoring @nest content
   let rval = {};
   _expandObject({
@@ -186,6 +234,7 @@ api.expand = ({
     expandedParent: rval,
     options,
     insideList,
+    typeScopedContext,
     expansionMap});
 
   // get property count on expanded output
@@ -352,7 +401,7 @@ function _expandObject({
     }
 
     // expand property
-    let expandedProperty = _expandIri(activeCtx, key, {vocab: true});
+    let expandedProperty = _expandIri(activeCtx, key, {vocab: true}, options);
 
     // drop non-absolute IRI keys that aren't keywords unless custom mapped
     if(expandedProperty === null ||
@@ -422,7 +471,7 @@ function _expandObject({
       _addValue(
         expandedParent, '@id',
         _asArray(value).map(v =>
-          _isString(v) ? _expandIri(activeCtx, v, {base: true}) : v),
+          _isString(v) ? _expandIri(activeCtx, v, {base: true}, options) : v),
         {propertyIsArray: options.isFrame});
       continue;
     }
@@ -433,7 +482,8 @@ function _expandObject({
         expandedParent, '@type',
         _asArray(value).map(v =>
           _isString(v) ?
-            _expandIri(activeCtx, v, {base: true, vocab: true}) : v),
+            _expandIri(activeCtx.previousContext || activeCtx, v,
+              {base: true, vocab: true}, options) : v),
         {propertyIsArray: options.isFrame});
       continue;
     }
@@ -555,15 +605,21 @@ function _expandObject({
     // use potential scoped context for key
     let termCtx = activeCtx;
     const ctx = _getContextValue(activeCtx, key, '@context');
-    if(ctx) {
-      termCtx = _processContext({activeCtx, localCtx: ctx, options});
+    if(!_isUndefined(ctx)) {
+      // Note: spec's `from term` var is named `isPropertyTermScopedContext`
+      termCtx = _processContext({
+        activeCtx,
+        localCtx: ctx,
+        isPropertyTermScopedContext: true,
+        options
+      });
     }
 
     const container = _getContextValue(termCtx, key, '@container') || [];
 
     if(container.includes('@language') && _isObject(value)) {
       // handle language map container (skip if value is not an object)
-      expandedValue = _expandLanguageMap(termCtx, value);
+      expandedValue = _expandLanguageMap(termCtx, value, options);
     } else if(container.includes('@index') && _isObject(value)) {
       // handle index container (skip if value is not an object)
       const asGraph = container.includes('@graph');
@@ -591,7 +647,8 @@ function _expandObject({
     } else if(container.includes('@type') && _isObject(value)) {
       // handle type container (skip if value is not an object)
       expandedValue = _expandIndexMap({
-        activeCtx: termCtx,
+        // since container is `@type`, revert type scoped context when expanding
+        activeCtx: termCtx.revertTypeScopedContext(),
         options,
         activeProperty: key,
         value,
@@ -671,7 +728,7 @@ function _expandObject({
 
     // FIXME: can this be merged with code above to simplify?
     // merge in reverse properties
-    if(termCtx.mappings[key] && termCtx.mappings[key].reverse) {
+    if(termCtx.mappings.has(key) && termCtx.mappings.get(key).reverse) {
       const reverseMap =
         expandedParent['@reverse'] = expandedParent['@reverse'] || {};
       expandedValue = _asArray(expandedValue);
@@ -703,7 +760,7 @@ function _expandObject({
     const nestedValues = _isArray(element[key]) ? element[key] : [element[key]];
     for(const nv of nestedValues) {
       if(!_isObject(nv) || Object.keys(nv).some(k =>
-        _expandIri(activeCtx, k, {vocab: true}) === '@value')) {
+        _expandIri(activeCtx, k, {vocab: true}, options) === '@value')) {
         throw new JsonLdError(
           'Invalid JSON-LD syntax; nested value must be a node object.',
           'jsonld.SyntaxError',
@@ -729,21 +786,23 @@ function _expandObject({
  * @param activeCtx the active context to use.
  * @param activeProperty the active property the value is associated with.
  * @param value the value to expand.
+ * @param {Object} [options] - processing options.
  *
  * @return the expanded value.
  */
-function _expandValue({activeCtx, activeProperty, value}) {
+function _expandValue({activeCtx, activeProperty, value, options}) {
   // nothing to expand
   if(value === null || value === undefined) {
     return null;
   }
 
   // special-case expand @id and @type (skips '@id' expansion)
-  const expandedProperty = _expandIri(activeCtx, activeProperty, {vocab: true});
+  const expandedProperty = _expandIri(
+    activeCtx, activeProperty, {vocab: true}, options);
   if(expandedProperty === '@id') {
-    return _expandIri(activeCtx, value, {base: true});
+    return _expandIri(activeCtx, value, {base: true}, options);
   } else if(expandedProperty === '@type') {
-    return _expandIri(activeCtx, value, {vocab: true, base: true});
+    return _expandIri(activeCtx, value, {vocab: true, base: true}, options);
   }
 
   // get type definition from context
@@ -751,11 +810,13 @@ function _expandValue({activeCtx, activeProperty, value}) {
 
   // do @id expansion (automatic for @graph)
   if((type === '@id' || expandedProperty === '@graph') && _isString(value)) {
-    return {'@id': _expandIri(activeCtx, value, {base: true})};
+    return {'@id': _expandIri(activeCtx, value, {base: true}, options)};
   }
   // do @id expansion w/vocab
   if(type === '@vocab' && _isString(value)) {
-    return {'@id': _expandIri(activeCtx, value, {vocab: true, base: true})};
+    return {
+      '@id': _expandIri(activeCtx, value, {vocab: true, base: true}, options)
+    };
   }
 
   // do not expand keyword values
@@ -789,14 +850,15 @@ function _expandValue({activeCtx, activeProperty, value}) {
  *
  * @param activeCtx the active context to use.
  * @param languageMap the language map to expand.
+ * @param {Object} [options] - processing options.
  *
  * @return the expanded language map.
  */
-function _expandLanguageMap(activeCtx, languageMap) {
+function _expandLanguageMap(activeCtx, languageMap, options) {
   const rval = [];
   const keys = Object.keys(languageMap).sort();
   for(const key of keys) {
-    const expandedKey = _expandIri(activeCtx, key, {vocab: true});
+    const expandedKey = _expandIri(activeCtx, key, {vocab: true}, options);
     let val = languageMap[key];
     if(!_isArray(val)) {
       val = [val];
@@ -827,11 +889,19 @@ function _expandIndexMap(
     indexKey}) {
   const rval = [];
   const keys = Object.keys(value).sort();
+  const isTypeIndex = indexKey === '@type';
   for(let key of keys) {
     // if indexKey is @type, there may be a context defined for it
-    const ctx = _getContextValue(activeCtx, key, '@context');
-    if(ctx) {
-      activeCtx = _processContext({activeCtx, localCtx: ctx, options});
+    if(isTypeIndex) {
+      const ctx = _getContextValue(activeCtx, key, '@context');
+      if(!_isUndefined(ctx)) {
+        activeCtx = _processContext({
+          activeCtx,
+          localCtx: ctx,
+          isTypeScopedContext: true,
+          options
+        });
+      }
     }
 
     let val = value[key];
@@ -840,11 +910,11 @@ function _expandIndexMap(
     }
 
     // expand for @type, but also for @none
-    const expandedKey = _expandIri(activeCtx, key, {vocab: true});
+    const expandedKey = _expandIri(activeCtx, key, {vocab: true}, options);
     if(indexKey === '@id') {
       // expand document relative
-      key = _expandIri(activeCtx, key, {base: true});
-    } else if(indexKey === '@type') {
+      key = _expandIri(activeCtx, key, {base: true}, options);
+    } else if(isTypeIndex) {
       key = expandedKey;
     }
 
@@ -854,6 +924,7 @@ function _expandIndexMap(
       element: val,
       options,
       insideList: false,
+      insideIndex: true,
       expansionMap
     });
     for(let item of val) {
