@@ -62,8 +62,7 @@ const {
 const {
   getInitialContext: _getInitialContext,
   process: _processContext,
-  getAllContexts: _getAllContexts,
-  expandIri: _expandIri
+  getAllContexts: _getAllContexts
 } = require('./context');
 
 const {
