@@ -45,8 +45,8 @@ if(typeof process === 'object' && typeof process.nextTick === 'function') {
 api.setImmediate = _setImmediate ? _delay : api.nextTick;
 
 /**
- * Clones an object, array, or string/number. If a typed JavaScript object
- * is given, such as a Date, it will be converted to a string.
+ * Clones an object, array, Map, Set, or string/number. If a typed JavaScript
+ * object is given, such as a Date, it will be converted to a string.
  *
  * @param value the value to clone.
  *
@@ -60,6 +60,16 @@ api.clone = function(value) {
       for(let i = 0; i < value.length; ++i) {
         rval[i] = api.clone(value[i]);
       }
+    } else if(value instanceof Map) {
+      rval = new Map();
+      for(const [k, v] of value) {
+        rval.set(k, api.clone(v));
+      }
+    } else if(value instanceof Set) {
+      rval = new Set();
+      for(const v of value) {
+        rval.add(api.clone(v));
+      }
     } else if(types.isObject(value)) {
       rval = {};
       for(const key in value) {
@@ -142,7 +152,7 @@ api.parseLinkHeader = header => {
     const rel = result['rel'] || '';
     if(Array.isArray(rval[rel])) {
       rval[rel].push(result);
-    } else if(rel in rval) {
+    } else if(rval.hasOwnProperty(rel)) {
       rval[rel] = [rval[rel], result];
     } else {
       rval[rel] = result;
@@ -192,7 +202,7 @@ api.validateTypeValue = v => {
  * @return true if the subject has the given property, false if not.
  */
 api.hasProperty = (subject, property) => {
-  if(property in subject) {
+  if(subject.hasOwnProperty(property)) {
     const value = subject[property];
     return (!types.isArray(value) || value.length > 0);
   }
@@ -253,13 +263,13 @@ api.addValue = (subject, property, value, options) => {
 
   if(types.isArray(value)) {
     if(value.length === 0 && options.propertyIsArray &&
-      !(property in subject)) {
+      !subject.hasOwnProperty(property)) {
       subject[property] = [];
     }
     for(let i = 0; i < value.length; ++i) {
       api.addValue(subject, property, value[i], options);
     }
-  } else if(property in subject) {
+  } else if(subject.hasOwnProperty(property)) {
     // check if subject already has value if duplicates not allowed
     const hasValue = (!options.allowDuplicate &&
       api.hasValue(subject, property, value));
