@@ -36,31 +36,48 @@ const TEST_TYPES = {
       // see JSON-LD 1.0 Errata
       specVersion: ['json-ld-1.0'],
       // FIXME
-      regex: [
+      idRegex: [
         // list of lists
-        /^#tli01/,
-        /^#tli02/,
-        /^#tli03/,
-        /^#tli04/,
-        /^#tli05/,
+        /compact-manifest.jsonld#tli01$/,
+        /compact-manifest.jsonld#tli02$/,
+        /compact-manifest.jsonld#tli03$/,
+        /compact-manifest.jsonld#tli04$/,
+        /compact-manifest.jsonld#tli05$/,
         // terms
-        /^#tp001/,
+        /compact-manifest.jsonld#tp001$/,
         // rel iri
-        /^#t0095/,
+        /compact-manifest.jsonld#t0095$/,
         // type set
-        /^#t0104/,
-        /^#t0105/,
+        /compact-manifest.jsonld#t0104$/,
+        /compact-manifest.jsonld#t0105$/,
         // rel vocab
-        /^#t0107/,
+        /compact-manifest.jsonld#t0107$/,
         // html
-        /^#th001/,
-        /^#th002/,
-        /^#th003/,
-        /^#th004/,
+        /compact-manifest.jsonld#th001$/,
+        /compact-manifest.jsonld#th002$/,
+        /compact-manifest.jsonld#th003$/,
+        /compact-manifest.jsonld#th004$/,
         // @type: @none
-        /^#ttn01/,
-        /^#ttn02/,
-        /^#ttn03/,
+        /compact-manifest.jsonld#ttn01$/,
+        /compact-manifest.jsonld#ttn02$/,
+        /compact-manifest.jsonld#ttn03$/,
+        // property-valued indexes
+        /compact-manifest.jsonld#tpi01$/,
+        /compact-manifest.jsonld#tpi02$/,
+        /compact-manifest.jsonld#tpi03$/,
+        /compact-manifest.jsonld#tpi04$/,
+        /compact-manifest.jsonld#tpi05$/,
+        /compact-manifest.jsonld#tpi06$/,
+        // JSON literals
+        /compact-manifest.jsonld#tjs01$/,
+        /compact-manifest.jsonld#tjs02$/,
+        /compact-manifest.jsonld#tjs03$/,
+        /compact-manifest.jsonld#tjs04$/,
+        /compact-manifest.jsonld#tjs05$/,
+        /compact-manifest.jsonld#tjs06$/,
+        /compact-manifest.jsonld#tjs07$/,
+        /compact-manifest.jsonld#tjs08$/,
+        /compact-manifest.jsonld#tjs09$/,
       ]
     },
     fn: 'compact',
@@ -74,68 +91,93 @@ const TEST_TYPES = {
   'jld:ExpandTest': {
     skip: {
       // FIXME
-      regex: [
+      idRegex: [
         // list of lists
-        /^#tli01/,
-        /^#tli02/,
-        /^#tli03/,
-        /^#tli04/,
-        /^#tli05/,
-        /^#tli06/,
-        /^#tli07/,
-        /^#tli08/,
-        /^#tli09/,
-        /^#tli10/,
+        /expand-manifest.jsonld#tli01$/,
+        /expand-manifest.jsonld#tli02$/,
+        /expand-manifest.jsonld#tli03$/,
+        /expand-manifest.jsonld#tli04$/,
+        /expand-manifest.jsonld#tli05$/,
+        /expand-manifest.jsonld#tli06$/,
+        /expand-manifest.jsonld#tli07$/,
+        /expand-manifest.jsonld#tli08$/,
+        /expand-manifest.jsonld#tli09$/,
+        /expand-manifest.jsonld#tli10$/,
         // mode
-        /^#tp001/,
-        /^#tp002/,
+        /expand-manifest.jsonld#tp001$/,
+        /expand-manifest.jsonld#tp002$/,
         // rel iri
-        /^#t0092/,
-        // remote
-        /^#t0005/,
-        /^#t0006/,
-        /^#t0007/,
-        /^#t0010/,
-        /^#t0011/,
-        /^#t0012/,
+        /expand-manifest.jsonld#t0092$/,
         // iris
-        /^#t0109/,
+        /expand-manifest.jsonld#t0109$/,
         // rel vocab
-        /^#t0110/,
-        /^#t0111/,
-        /^#t0112/,
+        /expand-manifest.jsonld#t0110$/,
+        /expand-manifest.jsonld#t0111$/,
+        /expand-manifest.jsonld#t0112$/,
         // html
-        /^#th001/,
-        /^#th002/,
-        /^#th003/,
-        /^#th004/,
-        /^#th005/,
-        /^#th006/,
-        /^#th007/,
-        /^#th010/,
-        /^#th011/,
-        /^#th012/,
-        /^#th013/,
-        /^#th014/,
-        /^#th015/,
-        /^#th016/,
-        /^#th017/,
-        /^#th018/,
-        /^#th019/,
-        /^#th020/,
-        /^#th021/,
-        /^#th022/,
-        // sealed
-        /^#tse01/,
-        /^#tse02/,
-        /^#tse03/,
-        /^#tse04/,
-        /^#tse05/,
-        /^#tse06/,
-        /^#tse08/,
-        /^#tse09/,
+        /expand-manifest.jsonld#th001$/,
+        /expand-manifest.jsonld#th002$/,
+        /expand-manifest.jsonld#th003$/,
+        /expand-manifest.jsonld#th004$/,
+        /expand-manifest.jsonld#th005$/,
+        /expand-manifest.jsonld#th006$/,
+        /expand-manifest.jsonld#th007$/,
+        /expand-manifest.jsonld#th010$/,
+        /expand-manifest.jsonld#th011$/,
+        /expand-manifest.jsonld#th012$/,
+        /expand-manifest.jsonld#th013$/,
+        /expand-manifest.jsonld#th014$/,
+        /expand-manifest.jsonld#th015$/,
+        /expand-manifest.jsonld#th016$/,
+        /expand-manifest.jsonld#th017$/,
+        /expand-manifest.jsonld#th018$/,
+        /expand-manifest.jsonld#th019$/,
+        /expand-manifest.jsonld#th020$/,
+        /expand-manifest.jsonld#th021$/,
+        /expand-manifest.jsonld#th022$/,
+        // HTML extraction
+        /expand-manifest.jsonld#thc01$/,
+        /expand-manifest.jsonld#thc02$/,
+        /expand-manifest.jsonld#thc03$/,
+        /expand-manifest.jsonld#thc04$/,
+        /expand-manifest.jsonld#thc05$/,
         // @type: @none
-        /^#ttn02/,
+        /expand-manifest.jsonld#ttn02$/,
+        // property index maps
+        /expand-manifest.jsonld#tpi01$/,
+        /expand-manifest.jsonld#tpi02$/,
+        /expand-manifest.jsonld#tpi03$/,
+        /expand-manifest.jsonld#tpi04$/,
+        /expand-manifest.jsonld#tpi05$/,
+        /expand-manifest.jsonld#tpi06$/,
+        /expand-manifest.jsonld#tpi07$/,
+        /expand-manifest.jsonld#tpi08$/,
+        /expand-manifest.jsonld#tpi09$/,
+        /expand-manifest.jsonld#tpi10$/,
+        /expand-manifest.jsonld#tpi11$/,
+        // JSON literals
+        /expand-manifest.jsonld#tjs01$/,
+        /expand-manifest.jsonld#tjs02$/,
+        /expand-manifest.jsonld#tjs03$/,
+        /expand-manifest.jsonld#tjs04$/,
+        /expand-manifest.jsonld#tjs05$/,
+        /expand-manifest.jsonld#tjs06$/,
+        /expand-manifest.jsonld#tjs07$/,
+        /expand-manifest.jsonld#tjs08$/,
+        /expand-manifest.jsonld#tjs09$/,
+        /expand-manifest.jsonld#tjs10$/,
+        /expand-manifest.jsonld#tjs11$/,
+        // misc
+        /expand-manifest.jsonld#te043$/,
+        /expand-manifest.jsonld#te044$/,
+        // remote
+        /remote-doc-manifest.jsonld#t0005$/,
+        /remote-doc-manifest.jsonld#t0006$/,
+        /remote-doc-manifest.jsonld#t0007$/,
+        /remote-doc-manifest.jsonld#t0010$/,
+        /remote-doc-manifest.jsonld#t0011$/,
+        /remote-doc-manifest.jsonld#t0012$/,
+        /remote-doc-manifest.jsonld#t0013$/,
       ]
     },
     fn: 'expand',
@@ -148,16 +190,16 @@ const TEST_TYPES = {
   'jld:FlattenTest': {
     skip: {
       // FIXME
-      regex: [
+      idRegex: [
         // list of lists
-        /^#tli01/,
-        /^#tli02/,
-        /^#tli03/,
+        /flatten-manifest.jsonld#tli01$/,
+        /flatten-manifest.jsonld#tli02$/,
+        /flatten-manifest.jsonld#tli03$/,
         // html
-        /^#th001/,
-        /^#th002/,
-        /^#th003/,
-        /^#th004/,
+        /flatten-manifest.jsonld#th001$/,
+        /flatten-manifest.jsonld#th002$/,
+        /flatten-manifest.jsonld#th003$/,
+        /flatten-manifest.jsonld#th004$/,
       ]
     },
     fn: 'flatten',
@@ -171,29 +213,36 @@ const TEST_TYPES = {
   'jld:FrameTest': {
     skip: {
       // FIXME
-      regex: [
+      idRegex: [
         // ex
-        /^#tg001/,
+        /frame-manifest.jsonld#tg001$/,
         // graphs
-        /^#t0010/,
-        /^#t0020/,
-        /^#t0046/,
-        /^#t0049/,
-        /^#t0051/,
-        /^#tg010/,
-        /^#tp046/,
-        /^#tp049/,
+        /frame-manifest.jsonld#t0010$/,
+        /frame-manifest.jsonld#t0020$/,
+        /frame-manifest.jsonld#t0046$/,
+        /frame-manifest.jsonld#t0049$/,
+        /frame-manifest.jsonld#t0051$/,
+        /frame-manifest.jsonld#tg010$/,
+        /frame-manifest.jsonld#tp046$/,
+        /frame-manifest.jsonld#tp049$/,
         // blank nodes
-        /^#t0052/,
-        /^#t0053/,
+        /frame-manifest.jsonld#t0052$/,
+        /frame-manifest.jsonld#t0053$/,
         // embed
-        /^#t0054/,
+        /frame-manifest.jsonld#t0054$/,
         // lists
-        /^#t0055/,
-        /^#t0058/,
+        /frame-manifest.jsonld#t0055$/,
+        /frame-manifest.jsonld#t0058$/,
         // misc
-        /^#tp010/,
-        /^#tp050/,
+        /frame-manifest.jsonld#tp010$/,
+        /frame-manifest.jsonld#tp050$/,
+        /frame-manifest.jsonld#teo01$/,
+        // @embed:@first
+        /frame-manifest.jsonld#t0060$/,
+        // requireAll
+        /frame-manifest.jsonld#tra01$/,
+        /frame-manifest.jsonld#tra02$/,
+        /frame-manifest.jsonld#tra03$/,
       ]
     },
     fn: 'frame',
@@ -207,11 +256,19 @@ const TEST_TYPES = {
   'jld:FromRDFTest': {
     skip: {
       // FIXME
-      regex: [
+      idRegex: [
         // list of lists
-        /^#tli01/,
-        /^#tli02/,
-        /^#tli03/,
+        /fromRdf-manifest.jsonld#tli01$/,
+        /fromRdf-manifest.jsonld#tli02$/,
+        /fromRdf-manifest.jsonld#tli03$/,
+        // JSON literals
+        /fromRdf-manifest.jsonld#tjs01$/,
+        /fromRdf-manifest.jsonld#tjs02$/,
+        /fromRdf-manifest.jsonld#tjs03$/,
+        /fromRdf-manifest.jsonld#tjs04$/,
+        /fromRdf-manifest.jsonld#tjs05$/,
+        /fromRdf-manifest.jsonld#tjs06$/,
+        /fromRdf-manifest.jsonld#tjs07$/,
       ]
     },
     fn: 'fromRDF',
@@ -232,24 +289,40 @@ const TEST_TYPES = {
   'jld:ToRDFTest': {
     skip: {
       // FIXME
-      regex: [
+      idRegex: [
         // list of lists
-        /^#tli01/,
-        /^#tli02/,
+        /toRdf-manifest.jsonld#tli01$/,
+        /toRdf-manifest.jsonld#tli02$/,
         // blank node properties
-        /^#t0118/,
+        /toRdf-manifest.jsonld#t0118$/,
         // well formed
-        /^#twf01/,
-        /^#twf02/,
-        /^#twf03/,
-        /^#twf04/,
-        /^#twf05/,
-        /^#twf06/,
-        /^#twf07/,
+        /toRdf-manifest.jsonld#twf01$/,
+        /toRdf-manifest.jsonld#twf02$/,
+        /toRdf-manifest.jsonld#twf03$/,
+        /toRdf-manifest.jsonld#twf04$/,
+        /toRdf-manifest.jsonld#twf05$/,
+        /toRdf-manifest.jsonld#twf06$/,
+        /toRdf-manifest.jsonld#twf07$/,
         // html
-        /^#th001/,
-        /^#th002/,
-        /^#th003/,
+        /toRdf-manifest.jsonld#th001$/,
+        /toRdf-manifest.jsonld#th002$/,
+        /toRdf-manifest.jsonld#th003$/,
+        // JSON literal
+        /toRdf-manifest.jsonld#tjs01$/,
+        /toRdf-manifest.jsonld#tjs02$/,
+        /toRdf-manifest.jsonld#tjs03$/,
+        /toRdf-manifest.jsonld#tjs04$/,
+        /toRdf-manifest.jsonld#tjs05$/,
+        /toRdf-manifest.jsonld#tjs06$/,
+        /toRdf-manifest.jsonld#tjs07$/,
+        /toRdf-manifest.jsonld#tjs08$/,
+        /toRdf-manifest.jsonld#tjs09$/,
+        /toRdf-manifest.jsonld#tjs10$/,
+        /toRdf-manifest.jsonld#tjs11$/,
+        /toRdf-manifest.jsonld#tjs12$/,
+        /toRdf-manifest.jsonld#tjs13$/,
+        // number fixes
+        /toRdf-manifest.jsonld#trt01$/,
       ]
     },
     fn: 'toRDF',
@@ -326,8 +399,8 @@ function _testsToMocha(tests) {
     }
     describe(suite.title, () => {
       suite.tests.forEach(test => {
-        if(test.skip) {
-          it.skip(test.title);
+        if(test.only) {
+          it.only(test.title, test.f);
           return;
         }
         it(test.title, test.f);
@@ -433,10 +506,16 @@ function addTest(manifest, test, tests) {
   test.manifest = manifest;
   const description = test_id + ' ' + (test.purpose || test.name);
 
-  tests.push({
+  const _test = {
     title: description,
     f: makeFn()
-  });
+  };
+  // only based on test manifest
+  // skip handled via skip()
+  if('only' in test) {
+    _test.only = test.only;
+  }
+  tests.push(_test);
 
   function makeFn() {
     return async function() {
@@ -444,26 +523,72 @@ function addTest(manifest, test, tests) {
       self.timeout(5000);
       const testInfo = TEST_TYPES[getJsonLdTestType(test)];
 
-      // skip unknown and explicitly skipped test types
+      // skip based on test manifest
+      if('skip' in test && test.skip) {
+        if(options.verboseSkip) {
+          console.log('Skipping test due to manifest:',
+            {id: test['@id'], name: test.name});
+        }
+        self.skip();
+      }
+
+      // skip based on unknown test type
       const testTypes = Object.keys(TEST_TYPES);
-      if(!isJsonLdType(test, testTypes) || isJsonLdType(test, SKIP_TESTS)) {
-        //const type = [].concat(
-        //  getJsonLdValues(test, '@type'),
-        //  getJsonLdValues(test, 'type')
-        //);
-        //console.log('Skipping test "' + test.name + '" of type: ' + type);
+      if(!isJsonLdType(test, testTypes)) {
+        if(options.verboseSkip) {
+          const type = [].concat(
+            getJsonLdValues(test, '@type'),
+            getJsonLdValues(test, 'type')
+          );
+          console.log('Skipping test due to unknown type:',
+            {id: test['@id'], name: test.name, type});
+        }
         self.skip();
       }
 
+      // skip based on test type
+      if(isJsonLdType(test, SKIP_TESTS)) {
+        if(options.verboseSkip) {
+          const type = [].concat(
+            getJsonLdValues(test, '@type'),
+            getJsonLdValues(test, 'type')
+          );
+          console.log('Skipping test due to test type:',
+            {id: test['@id'], name: test.name, type});
+        }
+        self.skip();
+      }
+
+      // skip based on type info
       if(testInfo.skip && testInfo.skip.type) {
-        //console.log('Skipping test "' + test.name + '" of type: ' + type);
+        if(options.verboseSkip) {
+          console.log('Skipping test due to type info:',
+            {id: test['@id'], name: test.name});
+        }
         self.skip();
       }
 
-      if(testInfo.skip && testInfo.skip.regex) {
-        testInfo.skip.regex.forEach(function(re) {
+      // skip based on id regex
+      if(testInfo.skip && testInfo.skip.idRegex) {
+        testInfo.skip.idRegex.forEach(function(re) {
+          if(re.test(test['@id'])) {
+            if(options.verboseSkip) {
+              console.log('Skipping test due to id:',
+                {id: test['@id']});
+            }
+            self.skip();
+          }
+        });
+      }
+
+      // skip based on description regex
+      if(testInfo.skip && testInfo.skip.descriptionRegex) {
+        testInfo.skip.descriptionRegex.forEach(function(re) {
           if(re.test(description)) {
-            //console.log('Skipping test "' + test.name + '" of description: ' + description);
+            if(options.verboseSkip) {
+              console.log('Skipping test due to description:',
+                {id: test['@id'], name: test.name, description});
+            }
             self.skip();
           }
         });
@@ -479,7 +604,10 @@ function addTest(manifest, test, tests) {
             skipModes = testInfo.skip.processingMode;
           }
           if(skipModes.indexOf(pm) !== -1) {
-            //console.log('Skipping test "' + test.name + '" of processing mode: ' + pm);
+            if(options.verboseSkip) {
+              console.log('Skipping test due to processingMode:',
+                {id: test['@id'], name: test.name, processingMode: pm});
+            }
             self.skip();
           }
         });
@@ -493,7 +621,10 @@ function addTest(manifest, test, tests) {
             skipVersions = testInfo.skip.specVersion;
           }
           if(skipVersions.indexOf(sv) !== -1) {
-            //console.log('Skipping test "' + test.name + '" for spec version: ' + sv);
+            if(options.verboseSkip) {
+              console.log('Skipping test due to specVersion:',
+                {id: test['@id'], name: test.name, specVersion: sv});
+            }
             self.skip();
           }
         });
